#!/usr/bin/env python3
# -*- coding: future_fstrings -*-

import datetime
import anybadge
from lighthouse_garden.utility import info, output, system
from lighthouse_garden.lighthouse import database, utility, interpreter


def fetch_data():
    """
    Fetch the lighthouse data of all given targets
    :return:
    """
    output.println(f'{output.Subject.INFO} Checking export path', verbose_only=True)
    system.check_path(f'{system.config["export_path"]}/{system.config["data_dir"]}')

    output.println(f'{output.Subject.INFO} Starting to process targets')
    for target in system.config['targets']:
        process_target(target)
        if 'subpages' in target:
            for subpage in target['subpages']:
                process_target(subpage)


def process_target(target):
    """
    Processing a single target entry
    :param target:
    :return:
    """
    _output_name = lighthouse(target)
    _result = interpreter.get_result_by_report_file(target, _output_name)
    if _result:
        database.set_data(target, _result)
        output.println(f'{output.Subject.OK} > {utility.get_parent_of_subpage(target)["title"] + " // " if utility.get_parent_of_subpage(target) else ""}{info.get_target_name(target)} ... {_result["performance"]}')
        utility.extend_html_report_with_info(_result, f'{utility.get_data_dir()}{_output_name}.report.html')
        generate_badges(target)
    else:
        utility.remove_file(f'{utility.get_data_dir()}{_output_name}.report.html')
    utility.remove_file(f'{utility.get_data_dir()}{_output_name}.report.json')


def lighthouse(target):
    """
    Fetch the lighthouse data of a specific target
    :param target: Dict
    :return:
    """
    output.println(f'{output.Subject.INFO} Fetching performance data for {info.get_target_name(target)}', verbose_only=True)

    _output_name = generate_output_name(target)
    _result = system.run_command(
        f'lighthouse {target["url"]} {build_options(_output_name)}', allow_fail=True
    )

    if not _result:
        output.println(f'{output.Subject.ERROR} > {info.get_target_name(target)} ...')
        system.config['errors'] = True

    return _output_name


def build_options(output_name):
    """
    Build the lighthouse options
    @ToDo: Use a separate config file?
    :param output_name: String
    :return:
    """
    _options = system.config['lighthouse']['options']
    _options += f' --chrome-flags="{system.config["lighthouse"]["chrome_flags"]}"'
    _options += f' --output json --output html --output-path {system.config["export_path"]}/{system.config["data_dir"]}' \
                f'{output_name}'
    return _options


def generate_output_name(target):
    """
    Generate a temporary output name, e.g. _google_28-12-2020_15-59
    :return:
    """
    return f'_{target["identifier"]}_{datetime.datetime.now().strftime("%d-%m-%Y_%H-%M")}'


def generate_badges(target):
    """
    Generate the badges for the various metrics of a specific target
    :param target: Dict
    :return:
    """
    output.println(f'{output.Subject.INFO} Generating badges', verbose_only=True)
    data = database.get_data(target)
    badges = {
        'performance': {
            'label': 'performance',
            'value': data['performance'] if 'performance' in data else '',
         },
        'accessibility': {
            'label': 'accessibility',
            'value': data['accessibility'] if 'accessibility' in data else '',
        },
        'best-practices': {
            'label': 'best-practices',
            'value': data['best-practices'] if 'best-practices' in data else '',
        },
        'seo': {
            'label': 'seo',
            'value': data['seo'] if 'seo' in data else '',
        },
        'average': {
            'label': 'performance',
            'value': data['average']['value'] if 'average' in data else '',
        }
    }

    for key, value in badges.items():
        generate_badge(
            target=target,
            layout=value,
            attribute=key
        )


def generate_badge(target, layout, attribute):
    """
    Generate a badge
    :param target: Dict
    :param layout: Dict
    :param attribute: String
    :return:
    """
    thresholds = {50: 'red',
                  90: 'yellow',
                  100: 'green'}
    badge = anybadge.Badge(layout['label'], round(layout['value']), thresholds=thresholds)

    badge.write_badge(f'{utility.get_data_dir()}_{target["identifier"]}.{attribute}.svg', overwrite=True)
